;;;;
;;;; Testing the reflective interpreter!
;;;;

;
; Simple lambdas for arithmetics...
;
((lambda (foo bar) (+ foo bar)) 1 2)
((lambda rest rest) foo bar 5)

;
; Set!
;
(set! b #t)
b
(set! show (lambda (m x) (display m) (write x) (newline) x))
(show "foo: " foo)
;
; simple macro
;
(set! x (macro () '(display "hello, world\n")))
(x)



;
; run-time macro
;
((lambda (y) (y)) x)

;
; Let's define the ``if'' special form from the boolean deconstructor...
;
(set! in-lambda (macro forms (append (list lambda '()) forms)))

(in-lambda (display "hello\n") foo)

(set! quoted (lambda (x) (list quote x)))
(set! mquoted (macro (x) (quoted (quoted x))))
(quoted 1)
(quoted foo)
(mquoted foo)
(mquoted (+ 1 2))

(set! if2
  (macro (test yes no)
     (list (list extensional-if
	    test
	    (list in-lambda yes)
	    (list in-lambda no)))))

(if2 b (show "yes" 1) (show "no" 0))

(set! if
  (macro (test yes . rest)
     (list (quoted if2) test yes (if2 (null? rest) #f (car rest)))))

'QUUX
(if b (show "TEST: " 'test))


(if b (show "OUI" 1) (show "NON" 0))

;
; let's test it!
;
((lambda (x y)
    (if #t (set! x 1) (set! y 2))
    (write (list x y))
    (newline)
    (+ x y))
 0 0)


(apply list
   1 2 3 '(4 5 6))


(set! define (macro (formals . body)
    ((lambda (cont)
       (if (pair? formals)
           (cont (car formals) (apply list lambda (cdr formals) body))
           (cont formals (list (apply list lambda '() body)))))
     (lambda (name definition)
       (list set! name definition)))))

(define x 3)
x
(define (f z) (+ z z 1))
(f x)


(set! map (lambda (f l)
   (if (pair? l)
       (cons (f (car l)) (map f (cdr l)))
       '())))

(set! 1+ (lambda (x) (+ x 1)))
(1+ 9)

(cons (1+ (car '(1 2))) 2)

(map write '(1 2 3))


(map 1+ '(3 4 5))
(map write '(1 2 3 4))

((lambda rest (map (lambda (x) (display x) (newline)) rest)) 1 2 3 4 5 6)


; we could go on and redefine a full Scheme interpreter,
; but first, we'd need a GC...
